/*** -*- Mode: Javascript; tab-width: 4;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Michael Nachbaur code.
The Initial Developer of the Original Code is Michael Nachbaur.

Copyright (C) 2005 Michael Nachbaur.  All Rights Reserved.

Contributor(s): Michael Nachbaur

***/

function TreeItem(aCells) {
    if (arguments.length > 0) this.init(aCells);
}
TreeItem.prototype = {
    DEBUG : 0,
    init : function(aCells) {
        if (this.DEBUG > 1) dump('= TreeItem.init("' + aCells + '")\n');
        this.mOpened = false;
        if (aCells)
            this.cells = aCells;
        else
            this.cells = new Array();    
        this.mTree = null;
        return;
    },
    rowIndex : null,
    get isContainer() {
        if (this.DEBUG > 1) dump('= get TreeItem.isContainer() = ' + Boolean(this.mTree != null) + '\n');
        return this.mTree != null;
    },
    set isContainer(aValue) {
        if (this.DEBUG > 1) dump('= set TreeItem.isContainer("' + aValue + '")\n');
        if (aValue && this.mTree == null) {
            this.mTree = new TreeView();
            this.mTree.parent = this;
        } else {
            this.mTree = null;
        }
    },
    get isContainerOpen() {
        if (this.DEBUG > 1) dump('= get TreeItem.isContainerOpen() = ' + this.mOpened + '\n');
        return this.mOpened;
    },
    get isContainerEmpty() {
        if (this.DEBUG > 1) dump('= get TreeItem.isContainerEmpty()\n');
        if (!this.isContainer)
            return false;
        return this.tree.rowCount == 0;
    },
    setCell : function(aCellName, aCell) {
        return this.cells[aCellName] = aCell;
    },
    getCell : function(aCell) {
        return this.cells[aCell];
    },
    cells : {},
    get rowCount() {
        if (this.DEBUG > 1) dump('= get TreeItem.rowCount()\n');
        var result;
        if (this.isContainer && this.isContainerOpen)
            result = this.mTree.rowCount + 1;
        else 
            result = 1;
        return result;
    },
    index : null,
    get parent() { return this.mParent; },
    set parent(aParent) { this.mParent = aParent; },
    get tree() {
        if (this.DEBUG > 1) dump('= get TreeItem.tree()\n');
        if (!this.isContainer)
            this.isContainer = true;
        return this.mTree;
    },
    set tree(aTree) {
        this.mTree = aTree;
    },
    toggleOpenState : function() {
        this.mOpened = !this.mOpened;
        return;
    },
    get nextSibling() {
        if (this.DEBUG > 1) dump('= TreeItem.nextSibling()\n');
        return this.parent.getItem(this.index + 1);
    },
    get previousSibling() {
        if (this.DEBUG > 1) dump('= TreeItem.previousSibling()\n');
        return this.parent.getItem(this.index - 1);
    },
    update: function() {
        if (this.DEBUG > 1) dump('= TreeItem.update()\n');
        this.parent.top.treeBox.rowCountChanged(this.rowIndex, 1);
        return;
    },

    // Row contents functions
    getImageSrc:     function(cell) {
        if (this.DEBUG > 1) dump('= TreeItem.getImageSrc("' + cell + '")\n');
        try {
            return this.getCell(cell).imageSrc;
        } catch (e) {}
        return null;
    },
    getProgressMode: function(cell) {
        if (this.DEBUG > 1) dump('= TreeItem.getProgressMode("' + cell + '")\n');
        try {
            var mode = this.getCell(cell).progressMode;
            if (mode == 'normal')
                return Components.interfaces.nsITreeView.progressNormal;
            else if (mode == 'undetermined')
                return Components.interfaces.nsITreeView.progressUndetermined;
            else if (mode == 'none')
                return Components.interfaces.nsITreeView.progressNone;
        } catch (e) {}
        return 0;
    },
    getCellValue:    function(cell) {
        if (this.DEBUG > 1) dump('= TreeItem.getCellValue("' + cell + '")\n');
        try {
            return this.getCell(cell).value;
        } catch (e) {}
        return null;
    },
    getCellText:     function(cell) {
        if (this.DEBUG > 1) dump('= TreeItem.getCellText("' + cell + '")\n');
        try {
            return this.getCell(cell).text;
        } catch (e) {}
        return null;
    },
    cycleCell:       function(cell) {
        if (this.DEBUG > 1) dump('= TreeItem.cycleCell("' + cell + '")\n');
        try {
            return this.getCell(cell).cycle;
        } catch (e) {}
        return;
    }
};

function TreeView() {
    this.init();
}
TreeView.prototype = {
    DEBUG : 0,
    init : function() {
        if (this.DEBUG > 1) dump('= TreeView.init()\n');
        this.mChildren = new Array();    
        this.mTop = this;
        return;
    },
    get wrappedJSObject() { return this; },
    mChildren : null,
    mTop : null,
    mParent : null,
    get top() { return this.mTop; },
    get children() { return this.mChildren; },
    get parent() { return this.mParent; },
    set parent(aParent) {
        this.mParent = aParent;
        var tree = aParent.parent;
        while (tree.parent) {
            tree = tree.parent.parent;
        }
        this.mTop = tree;
    },
    appendItem : function (aItem) {
        if (this.DEBUG > 1) dump('= TreeView.appendItem("' + aItem + '")\n');
        this.mChildren.push(aItem);
        aItem.index = this.mChildren.length - 1;
        aItem.parent = this;
        this.update(aItem, 1);
        return;
    },
    createItem : function (aCells) {
        if (this.DEBUG > 1) dump('= TreeView.createItem("' + aCells + '")\n');
        var item = new TreeItem(aCells);
        this.appendItem(item);
        return item;
    },
    removeItem : function (index) {
        if (this.DEBUG > 1) dump('= TreeView.getItem("' + index + '")\n');
        if (index < 0 || index >= this.mChildren.length)
            return null;
        var removedItem = this.mChildren.splice(index, 1);
        this.update();
        return removedItem;
    },
    removeItemByRow : function (row) {
        if (this.DEBUG > 1) dump('= TreeView.getItem("' + index + '")\n');
        if (index < 0 || index >= this.mChildren.length)
            return null;
        var rowItem = this.top.mSortedRows[row];
        var removedItem = rowItem.parent.mChildren.splice(rowItem.index, 1);
        this.update();
        return removedItem;
    },
    getItem : function (index) {
        if (this.DEBUG > 1) dump('= TreeView.getItem("' + index + '")\n');
        if (index < 0 || index >= this.mChildren.length)
            return null;
        return this.mChildren[index];
    },
    getItemByRow : function (row) {
        if (this.DEBUG > 1) dump('= TreeView.getItemByRow(' + row + ')\n');
        return this.top.mSortedRows[row];
    },
    update : function(aItem, aRowCount) {
        var top = this.top;
        top.updateRowIndexes();
        if (aItem) {
            if (this.DEBUG > 1) dump("Updating row count for " + aItem.rowIndex + "\n");
            try {
                top.treeBox.rowCountChanged(aItem.rowIndex, aRowCount);
            } catch(e) {
                dump("Caught exception: " + e + "\n");
            }
        }
        return;
    },
    updateRowIndexes : function() {
        if (this.DEBUG > 1) dump('= TreeView.updateRowIndexes()\n');
        var rowCount = 0;
        // Load the list of our immediate children, from the root of the tree
        var children = new Array();
        this.top.mSortedRows = new Array();
        for (var idx = 0; idx < this.top.children.length; idx++)
            children.push(this.top.children[idx]);

        // Iterate through our available children
        while (children.length > 0) {
            // Add this child as the first item
            var item = children.shift();
            this.top.mSortedRows.push(item);
            item.rowIndex = rowCount++;

            // If this is a container, prepend its children onto our stack
            if (item.isContainer && item.isContainerOpen) {
                var treeChildren = item.tree.children;
                for (var idx = treeChildren.length - 1; idx >= 0; idx--)
                    children.unshift(treeChildren[idx]);
            }
        }
        return;
    },

    // nsISupports
    QueryInterface : function(aIID) {
        if (Components.interfaces.nsIClassInfo.equals(aIID))
            return TreeView.prototype;
        if (Components.interfaces.nsITreeView.equals(aIID) ||
                Components.interfaces.nsISupportsWeakReference.equals(aIID) ||
                Components.interfaces.nsISupports.equals(aIID))
            return this;
        throw 0x80004002; // Components.results.NS_NOINTERFACE;
    },

    // nsIClassInfo
    getInterfaces: function(count) {
        count.value = 4;
        return [
            Components.interfaces.nsITreeView,
            Components.interfaces.nsIClassInfo, 
            Components.interfaces.nsISupportsWeakReference, 
            Components.interfaces.nsISupports
        ];
    },
    getHelperForLanguage: function(language) { return null; },
    get contractID() { return null; },
    get classDescription() { return "nsTreeView"; },
    get classID() { return null; },
    get implementationLanguage() { return Components.interfaces.nsIProgrammimgLanguage.JAVASCRIPT; },
    get flags() { return Components.interfaces.nsIClassInfo.DOM_OBJECT; },

    // nsITreeView
    get rowCount() {
        if (this.DEBUG > 1) dump('= get TreeView.rowCount()\n');
        if (!this.top)
            return this.mSortedRows.length;

        var count = 0;
        for (var idx = 0; idx < this.mChildren.length; idx++) {
            count += this.mChildren[idx].rowCount;
        }
        return count;
    },

    // Basic functions
    selection: null,
    treeBox: null,
    setTree: function(treeBox) {
        if (this.DEBUG > 1) dump('= TreeView.setTree(' + treeBox + ')\n');
        this.treeBox = treeBox;
        if (!treeBox)
            this.selection = null;
        if (this.DEBUG > 1) dump('  this.treeBox = "' + this.treeBox + '"\n');
        return;
    },

    // Display Properties
    getRowProperties:    function(row, prop) { },
    getCellProperties:   function(row, column, prop) { },
    getColumnProperties: function(column, elem, prop) { },
    isSeparator:         function(row) { return false; },
    isEditable:          function(row, column) { return false; },

    // Heirarchial Functions
    isContainer:         function(row) {
        if (this.DEBUG > 1) dump('= TreeView.isContainer(' + row + ')\n');
        return this.getItemByRow(row).isContainer;
    },
    isContainerOpen:     function(row) {
        if (this.DEBUG > 1) dump('= TreeView.isContainerOpen(' + row + ')\n');
        return this.getItemByRow(row).isContainerOpen;
    },
    isContainerEmpty:    function(row) {
        if (this.DEBUG > 1) dump('= TreeView.isContainerEmpty(' + row + ')\n');
        return this.getItemByRow(row).isContainerEmpty;
    },
    toggleOpenState:     function(row) {
        if (this.DEBUG > 1) dump('= TreeView.toggleOpenState(' + row + ')\n');
        var item = this.getItemByRow(row);
        item.toggleOpenState();
        this.update();
        var changedRows = item.tree.rowCount;
        if (!item.isContainerOpen)
            changedRows *= -1;
        this.top.treeBox.rowCountChanged(row, changedRows);
        return;
    },
    getParentIndex:      function(row) {
        if (this.DEBUG > 1) dump('= TreeView.getParentIndex(' + row + ')\n');
        return this.getItemByRow(row).parent.parent.rowIndex;
    },
    hasNextSibling:      function(row, after) {
        if (this.DEBUG > 1) dump('= TreeView.hasNextSibling(' + row + ', ' + after + ')\n');
        var item = this.getItemByRow(row);
        if (item.parent.mChildren.length > item.index + 1)
            return true;
        else
            return false;
    },
    getLevel:            function(row) {
        if (this.DEBUG > 1) dump('= TreeView.getLevel(' + row + ')\n');
        var item = this.getItemByRow(row).parent;
        var level = 0;
        while (item.parent != null) {
            level++;
            item = item.parent.parent;
        }
        return level;
    },

    // Row contents functions
    getImageSrc:         function(row, column) {
        if (this.DEBUG > 1) dump('= TreeView.getImageSrc(' + row + ', "' + column + '")\n');
        return this.getItemByRow(row).getImageSrc(column);
    },
    getProgressMode :    function(row, column) {
        if (this.DEBUG > 1) dump('= TreeView.getProgressMode(' + row + ', "' + column + '")\n');
        return this.getItemByRow(row).getProgressMode(column);
    },
    getCellValue:        function(row, column) {
        if (this.DEBUG > 1) dump('= TreeView.getCellValue(' + row + ', "' + column + '")\n');
        return this.getItemByRow(row).getCellValue(column);
    },
    getCellText:         function(row, column) {
        if (this.DEBUG > 1) dump('= TreeView.getCellText(' + row + ', "' + column + '")\n');
        return this.getItemByRow(row).getCellText(column);
    },
    cycleCell:           function(row, column) {
        if (this.DEBUG > 1) dump('= TreeView.cycleCell(' + row + ', "' + column + '")\n');
        return this.getItemByRow(row).cycleCell(column);
    },

    // Sorting
    isSorted:            function() { return false; },
    cycleHeader:         function(col, elem) { },

    // Selection
    selectionChanged:    function() { },

    // Drag & Drop
    canDropOn:           function(row) { return false; },
    canDropBeforeAfter:  function(row, before) { return false; },
    drop:                function(row, orientation) { },

    // Commands
    performAction:       function(action) { },
    performActionOnCell: function(action, row, column) { },
};
